@@ -1,6 +1,6 @@
 
 // balance - a balancing tcp proxy
-// $Revision: 2.24 $
+// $Revision: 2.28 $
 //
 // Copyright (c) 2000,2001 by Thomas Obermair (obermair@acm.org) 
 // and Inlab Software GmbH (info@inlab.de), Gruenwald, Germany. 
@@ -17,6 +17,13 @@
 //
 // 2.24:
 //   'channel 2 overload' problem fixed, thanks to Ed "KuroiNeko"
+// 2.26:
+//   'endless loop error' fixed, thanks to Anthony Baxter
+// 2.27:
+//   strcmp on NULL removed, thanks to Jay. D. Allen
+// 2.28: 
+//   bsent and breceived now unsigned to avoid nagative values,
+//   thanks to Anthony Baxter
 //
 
 #include <stdio.h>
@@ -64,8 +71,8 @@
 #define MAXINPUTLINE 	128	// max line in input mode 
 #define DEFAULTTIMEOUT  5	// default-timeout for non reachable hosts
 
-static char* rcsid="$Id: balance.c,v 2.24 2000/12/25 12:48:37 tommy Exp $";
-static char* revision="$Revision: 2.24 $";
+static char* rcsid="$Id: balance.c,v 2.28 2001/05/31 06:00:30 tommy Exp $";
+static char* revision="$Revision: 2.28 $";
 
 static char rendezvousfile[FILENAMELEN];
 static int  rendezvousfd;	
@@ -76,8 +83,8 @@ typedef struct {
   struct in_addr ipaddr;
   int c;			// current # of connections
   int maxc;			// max # of connections, 0 = no limit
-  int bsent;			// bytes sent
-  int breceived;		// bytes received
+  unsigned int bsent;		// bytes sent
+  unsigned int breceived;	// bytes received
 } CHANNEL;
 
 typedef struct {
@@ -312,9 +319,11 @@ int setaddress(struct in_addr *ipaddr, int *port, char* string, int default_port
     port_string="";
   }
 
-  if(!strcmp(port_string,"")) 
+  // fix for RedHat 7.0/7.1 choke on strcmp with NULL
+
+  if(port_string != NULL && !strcmp(port_string,"")) 
     port_string=NULL; 
-  if(!strcmp(maxc_string,"")) 
+  if(maxc_string != NULL && !strcmp(maxc_string,"")) 
     maxc_string=NULL; 
   
   hent = gethostbyname(string);
@@ -581,6 +590,9 @@ again:
             } else {
               if(grp_nchannels(common,groupindex) > 0) {
                 index=grp_current(common,groupindex);
+                startindex=index; 	// This fixes the "endless loop error"
+					// with all hosts being down and one
+					// in the last group... (from Anthony Baxter) 
               } else {
                 goto again;
               }
@@ -809,11 +821,11 @@ shell() {
           {
             int group;
 
-	    printf("%3s %2s %3s %16s %5s %4s %4s %10s %10s\n",
+	    printf("%3s %2s %3s %16s %5s %4s %4s %11s %11s\n",
 		   "GRP", "#", "S","ip-address","port","c","maxc","sent","rcvd"); 
             for (group=0; group<=MAXGROUPS; group++) {
 	      for(i=0; i<grp_nchannels(common,group); i++) {
-	        printf("%3d %2d %3s %16s %5d %4d %4d %10d %10d\n", 
+	        printf("%3d %2d %3s %16s %5d %4d %4d %11u %11u\n", 
                        group,
 		       i, 
 		       chn_status(common,group,i)==1?"ENA":"dis",
@@ -959,7 +971,7 @@ shell() {
         b_unlock();
 
       } else if(mycmp(command,"version")) {
-	printf("  $Id: balance.c,v 2.24 2000/12/25 12:48:37 tommy Exp $\n");
+	printf("  $Id: balance.c,v 2.28 2001/05/31 06:00:30 tommy Exp $\n");
         printf("  MAXGROUPS=%d\n", MAXGROUPS);
         printf("  MAXCHANNELS=%d\n", MAXCHANNELS);
       } else {
@@ -1154,7 +1166,7 @@ char	*argv[];
 
   common=makecommon(argc,argv,source_port);
 
-  listen(sockfd, 5);
+  listen(sockfd, SOMAXCONN);
 
 #ifdef MYBSD
   act.sa_handler = sig_chld;
